const hello = "Hello!";

export {hello};

export function supportsLocalStorage() {
	try {
		localStorage.setItem('test', 'test');
		localStorage.removeItem('test');

		return true;
	} catch(error) {
		return false;
	}
}

export function $(selector) {
	return document.querySelector(selector);
}

export function $$(selector) {
	return document.querySelectorAll(selector);
}

export class Component {
	$el;

	constructor(rootSelector) {
		this.$el = $(rootSelector);
	}

	$(selector) {
		return this.$el.querySelector(selector);
	}

	$$(selector) {
		return this.$el.querySelectorAll(selector);
	}
}

/**
 * @function doXHR helper to perform an XMLHTTPRequest
 * @param {string} method the HTTP method
 * @param {string} url the url to call
 * @param {any} data the data to sendm if any
 * @param {Object} headers the key/value for headers
 * @param {function} cb the callback after async operation concludes, it will include
 * @returns {Promise}
 */
 export function doXHR(method, url, data, headers) {
	return new Promise(function (resolve, reject) {
		const xhr = new XMLHttpRequest();
		xhr.open(method, url);
		xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
		headers = headers || {};
		Object.keys(headers).forEach((h) => {
			xhr.setRequestHeader(h, headers[h]);
		});
		xhr.addEventListener("load", () => resolve(xhr));
		xhr.addEventListener("error", () => reject(xhr));
		xhr.send(data);
	});
}
